(*********************************************************************
*
* WTSAPI32.H
*
*   Windows Terminal Server public APIs
*
*   Copyright (c) 1997-2001 Microsoft Corporation
*
**********************************************************************)

{ Adapted for Free Pascal by Michael Malicoat <michael@circusmachina.com>.

  This file is part of ASE, the Abandonded Sessions Eliminator.

  This program is free software: you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 3 of the License or, at your option, any later
  version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABLITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.

  You should have received a copy of the GNU General Public License along with
  this program.  If not, see <http://www.gnu.org/licenses/>.
}
unit wtsapi32;

{$mode objfpc}{$H+}

interface

uses
  Windows;

const
  wtsapi32_dll = 'wtsapi32.dll';

// These are not defined in the FPC Windows unit for some reason
type PLPSTR = ^LPSTR;
type PLPWSTR = ^LPWSTR;
type PWSTR = LPWSTR;
type PPVOID = ^PVOID;

(*===================================================================
==   Defines
=====================================================================*)

const
  (*
   *  Specifies the current server
   *)
  WTS_CURRENT_SERVER = THandle(0);
  WTS_CURRENT_SERVER_HANDLE = THandle(0);
  WTS_CURRENT_SERVER_NAME = nil;

  (*
   *  Specifies the current session (SessionId)
   *)
  WTS_CURRENT_SESSION = DWORD(-1);

  (*
   *  Specifies any-session (SessionId)
   *)
  WTS_ANY_SESSION = DWORD(-2);

  (*
   *  Possible pResponse values from WTSSendMessage()
   *)
  IDTIMEOUT = 32000;
  IDASYNC = 32001;

  USERNAME_LENGTH = 20;
  CLIENTNAME_LENGTH = 20;
  CLIENTADDRESS_LENGTH = 30;

  (*
   *  Shutdown flags
   *)
  WTS_WSD_LOGOFF          = $00000001;  // log off all users except
                                        // current user; deletes
                                        // WinStations (a reboot is
                                        // required to recreate the
                                        // WinStations)

  WTS_WSD_SHUTDOWN        = $00000002;  // shutdown system

  WTS_WSD_REBOOT          = $00000004;  // shutdown and reboot

  WTS_WSD_POWEROFF        = $00000008;  // shutdown and power off (on
                                        // machines that support power
                                        // off through software)

  WTS_WSD_FASTREBOOT      = $00000010;  // reboot without logging users
                                        // off or shutting down

  MAX_ELAPSED_TIME_LENGTH = 15;
  MAX_DATE_TIME_LENGTH    = 56;
  WINSTATIONNAME_LENGTH   = 32;
  DOMAIN_LENGTH           = 17;

  WTS_DRIVE_LENGTH         = 3;
  WTS_LISTENER_NAME_LENGTH = 32;
  WTS_COMMENT_LENGTH       = 60;

  (*
   *  Flags for WTSCreateListener
   *)
  WTS_LISTENER_CREATE = $00000001;
  WTS_LISTENER_UPDATE = $00000010;

  (*********************************
   *   Listener access values
   *********************************)
   WTS_SECURITY_QUERY_INFORMATION        = $00000001;
   WTS_SECURITY_SET_INFORMATION          = $00000002;
   WTS_SECURITY_RESET                    = $00000004;
   WTS_SECURITY_VIRTUAL_CHANNELS         = $00000008;
   WTS_SECURITY_REMOTE_CONTROL           = $00000010;
   WTS_SECURITY_LOGON                    = $00000020;
   WTS_SECURITY_LOGOFF                   = $00000040;
   WTS_SECURITY_MESSAGE                  = $00000080;
   WTS_SECURITY_CONNECT                  = $00000100;
   WTS_SECURITY_DISCONNECT               = $00000200;

   WTS_SECURITY_GUEST_ACCESS             = (WTS_SECURITY_LOGON);

   WTS_SECURITY_CURRENT_GUEST_ACCESS     = (WTS_SECURITY_VIRTUAL_CHANNELS or WTS_SECURITY_LOGOFF);

   WTS_SECURITY_USER_ACCESS              = (WTS_SECURITY_CURRENT_GUEST_ACCESS or WTS_SECURITY_QUERY_INFORMATION or WTS_SECURITY_CONNECT );

   WTS_SECURITY_CURRENT_USER_ACCESS      = (WTS_SECURITY_SET_INFORMATION or WTS_SECURITY_RESET or
                                            WTS_SECURITY_VIRTUAL_CHANNELS or WTS_SECURITY_LOGOFF or
                                            WTS_SECURITY_DISCONNECT);

   WTS_SECURITY_ALL_ACCESS               = (STANDARD_RIGHTS_REQUIRED or WTS_SECURITY_QUERY_INFORMATION or
                                            WTS_SECURITY_SET_INFORMATION or WTS_SECURITY_RESET or
                                            WTS_SECURITY_VIRTUAL_CHANNELS or WTS_SECURITY_REMOTE_CONTROL or
                                            WTS_SECURITY_LOGON or
                                            WTS_SECURITY_MESSAGE or
                                            WTS_SECURITY_CONNECT or WTS_SECURITY_DISCONNECT);

(*===================================================================
==   WTS_CONNECTSTATE_CLASS - Session connect state
=====================================================================*)

type WTS_CONNECTSTATE_CLASS = (
  WTSActive,              // User logged on to WinStation
  WTSConnected,           // WinStation connected to client
  WTSConnectQuery,        // In the process of connecting to client
  WTSShadow,              // Shadowing another WinStation
  WTSDisconnected,        // WinStation logged on without client
  WTSIdle,                // Waiting for client to connect
  WTSListen,              // WinStation is listening for connection
  WTSReset,               // WinStation is being reset
  WTSDown,                // WinStation is down due to error
  WTSInit                 // WinStation in initialization
);

type PWTS_CONNECTSTATE_CLASS = ^WTS_CONNECTSTATE_CLASS;

(*=====================================================================
==   WTS_SERVER_INFO - returned by WTSEnumerateServers (version 1)
=====================================================================*)

(*
 *  WTSEnumerateServers() returns two variables: pServerInfo and Count.
 *  The latter is the number of WTS_SERVER_INFO structures contained in
 *  the former.  In order to read each server, iterate i from 0 to
 *  Count-1 and reference the server name as
 *  pServerInfo[i].pServerName; for example:
 *
 *  for ( i=0; i < Count; i++ ) {
 *      _tprintf( TEXT("%s "), pServerInfo[i].pServerName );
 *  }
 *
 *  The memory returned looks like the following.  P is a pServerInfo
 *  pointer, and D is the string data for that pServerInfo:
 *
 *  P1 P2 P3 P4 ... Pn D1 D2 D3 D4 ... Dn
 *
 *  This makes it easier to iterate the servers, using code similar to
 *  the above.
 *)
type
  PPWTS_SERVER_INFOW = ^PWTS_SERVER_INFOW;
  PWTS_SERVER_INFOW = ^WTS_SERVER_INFOW;
  WTS_SERVER_INFOW = record
    pServerName: LPWSTR;    // server name
  end;

type
  PPWTS_SERVER_INFOA = ^PWTS_SERVER_INFOA;
  PWTS_SERVER_INFOA = ^WTS_SERVER_INFOA;
  WTS_SERVER_INFOA = record
    pServerName: LPSTR;    // server name
  end;

{$IFDEF UNICODE}
type WTS_SERVER_INFO = WTS_SERVER_INFOW;
{$ELSE}
type WTS_SERVER_INFO = WTS_SERVER_INFOA;
{$ENDIF}

type PWTS_SERVER_INFO = ^WTS_SERVER_INFO;
type PPWTS_SERVER_INFO = ^PWTS_SERVER_INFO;

(*=====================================================================

==   WTS_SESSION_INFO - returned by WTSEnumerateSessions (version 1)

=====================================================================*)

(*
 *  WTSEnumerateSessions() returns data in a similar format to the above
 *  WTSEnumerateServers().  It returns two variables: pSessionInfo and
 *  Count.  The latter is the number of WTS_SESSION_INFO structures
 *  contained in the former.  Iteration is similar, except that there
 *  are three parts to each entry, so it would look like this:
 *
 *  for ( i=0; i < Count; i++ ) {
 *      _tprintf( TEXT("%-5u  %-20s  %u\n"),
                  pSessionInfo[i].SessionId,
 *                pSessionInfo[i].pWinStationName,
 *                pSessionInfo[i].State );
 *  }
 *
 *  The memory returned is also segmented as the above, with all the
 *  structures allocated at the start and the string data at the end.
 *  We'll use S for the SessionId, P for the pWinStationName pointer
 *  and D for the string data, and C for the connect State:
 *
 *  S1 P1 C1 S2 P2 C2 S3 P3 C3 S4 P4 C4 ... Sn Pn Cn D1 D2 D3 D4 ... Dn
 *
 *  As above, this makes it easier to iterate the sessions.
 *)

type
  PPWTS_SESSION_INFOW = ^PWTS_SESSION_INFOW;
  PWTS_SESSION_INFOW = ^WTS_SESSION_INFOW;
  WTS_SESSION_INFOW = record
    SessionId: DWORD;              // session id
    pWinStationName: LPWSTR;       // name of WinStation this session is
                                   // connected to
    State: WTS_CONNECTSTATE_CLASS; // connection state (see enum)
  end;

type
  PPWTS_SESSION_INFOA = ^PWTS_SESSION_INFOA;
  PWTS_SESSION_INFOA = ^WTS_SESSION_INFOA;
  WTS_SESSION_INFOA = record
    SessionId: DWORD;              // session id
    pWinStationName: LPSTR;        // name of WinStation this session is
                                   // connected to
    State: WTS_CONNECTSTATE_CLASS; // connection state (see enum)
  end;

type
  PPWTS_SESSION_INFO_1W = ^PWTS_SESSION_INFO_1W;
  PWTS_SESSION_INFO_1W = ^WTS_SESSION_INFO_1W;
  WTS_SESSION_INFO_1W = record
    ExecEnvId: DWORD;
    State: WTS_CONNECTSTATE_CLASS;
    SessionId: DWORD;
    pSessionName: LPWSTR;
    pHostName: LPWSTR;
    pUserName: LPWSTR;
    pDomainName: LPWSTR;
    pFarmName: LPWSTR;
  end;

type
  PPWTS_SESSION_INFO_1A = ^PWTS_SESSION_INFO_1A;
  PWTS_SESSION_INFO_1A = ^WTS_SESSION_INFO_1A;
  WTS_SESSION_INFO_1A = record
    ExecEnvId: DWORD;
    State: WTS_CONNECTSTATE_CLASS;
    SessionId: DWORD;
    pSessionName: LPSTR;
    pHostName: LPSTR;
    pUserName: LPSTR;
    pDomainName: LPSTR;
    pFarmName: LPSTR;
  end;

{$IFDEF UNICODE}
type WTS_SESSION_INFO = WTS_SESSION_INFOW;
type WTS_SESSION_INFO_1 = WTS_SESSION_INFO_1W;
{$ELSE}
type WTS_SESSION_INFO = WTS_SESSION_INFOA;
type WTS_SESSION_INFO_1 = WTS_SESSION_INFO_1A;
{$ENDIF}

type PWTS_SESSION_INFO = ^WTS_SESSION_INFO;
type PPWTS_SESSION_INFO = ^PWTS_SESSION_INFO;
type PWTS_SESSION_INFO_1 = ^WTS_SESSION_INFO_1;
type PPWTS_SESSION_INFO_1 = ^PWTS_SESSION_INFO_1;

(*=====================================================================
==   WTS_PROCESS_INFO - returned by WTSEnumerateProcesses (version 1)
=====================================================================*)

(*
 *  WTSEnumerateProcesses() also returns data similar to
 *  WTSEnumerateServers().  It returns two variables: pProcessInfo and
 *  Count.  The latter is the number of WTS_PROCESS_INFO structures
 *  contained in the former.  Iteration is similar, except that there
 *  are four parts to each entry, so it would look like this:
 *
 *  for ( i=0; i < Count; i++ ) {
 *      GetUserNameFromSid( pProcessInfo[i].pUserSid, UserName,
 *                          sizeof(UserName) );
 *      _tprintf( TEXT("%-5u  %-20s  %-5u  %s\n"),
 *              pProcessInfo[i].SessionId,
 *              UserName,
 *              pProcessInfo[i].ProcessId,
 *              pProcessInfo[i].pProcessName );
 *  }
 *
 *  The memory returned is also segmented as the above, with all the
 *  structures allocated at the start and the string data at the end.
 *  We'll use S for the SessionId, R for the ProcessId, P for the
 *  pProcessName pointer and D for the string data, and U for pUserSid:
 *
 *  S1 R1 P1 U1 S2 R2 P2 U2 S3 R3 P3 U3 ... Sn Rn Pn Un D1 D2 D3 ... Dn
 *
 *  As above, this makes it easier to iterate the processes.
 *)

type
  PPWTS_PROCESS_INFOW = ^PWTS_PROCESS_INFOW;
  PWTS_PROCESS_INFOW = ^WTS_PROCESS_INFOW;
  WTS_PROCESS_INFOW = record
    SessionId: DWORD;      // session id
    ProcessId: DWORD;      // process id
    pProcessName: LPWSTR;  // name of process
    pUserSid: PSID;        // user's SID
  end;

type
  PPWTS_PROCESS_INFOA = ^PWTS_PROCESS_INFOA;
  PWTS_PROCESS_INFOA = ^WTS_PROCESS_INFOA;
  WTS_PROCESS_INFOA = record
    SessionId: DWORD;      // session id
    ProcessId: DWORD;      // process id
    pProcessName: LPSTR;   // name of process
    pUserSid: PSID;        // user's SID
  end;

{$IFDEF UNICODE}
type WTS_PROCESS_INFO = WTS_PROCESS_INFOW;
{$ELSE}
type WTS_PROCESS_INFO = WTS_PROCESS_INFOA;
{$ENDIF}

type PWTS_PROCESS_INFO = ^WTS_PROCESS_INFO;
type PPWTS_PROCESS_INFO = ^PWTS_PROCESS_INFO;

(*=====================================================================
==   WTS_INFO_CLASS - WTSQuerySessionInformation
==    (See additional typedefs for more info on structures)
=====================================================================*)

const
  WTS_PROTOCOL_TYPE_CONSOLE        = 0;    // Console
  WTS_PROTOCOL_TYPE_ICA            = 1;    // ICA Protocol
 WTS_PROTOCOL_TYPE_RDP             = 2;    // RDP Protocol

type WTS_INFO_CLASS = (
  WTSInitialProgram         = 0,
  WTSApplicationName        = 1,
  WTSWorkingDirectory       = 2,
  WTSOEMId                  = 3,
  WTSSessionId              = 4,
  WTSUserName               = 5,
  WTSWinStationName         = 6,
  WTSDomainName             = 7,
  WTSConnectState           = 8,
  WTSClientBuildNumber      = 9,
  WTSClientName             = 10,
  WTSClientDirectory        = 11,
  WTSClientProductId        = 12,
  WTSClientHardwareId       = 13,
  WTSClientAddress          = 14,
  WTSClientDisplay          = 15,
  WTSClientProtocolType     = 16,
  WTSIdleTime               = 17,
  WTSLogonTime              = 18,
  WTSIncomingBytes          = 19,
  WTSOutgoingBytes          = 20,
  WTSIncomingFrames         = 21,
  WTSOutgoingFrames         = 22,
  WTSClientInfo             = 23,
  WTSSessionInfo            = 24,
  WTSSessionInfoEx          = 25,
  _WTSConfigInfo            = 26,
  WTSValidationInfo         = 27,  // Info Class value used to fetch Validation Information through the WTSQuerySessionInformation
  WTSSessionAddressV4       = 28,
  WTSIsRemoteSession        = 29
);

(*=====================================================================
==   WTS Config Information
=====================================================================*)
type
  PWTSCONFIGINFOW = ^WTSCONFIGINFOW;
  WTSCONFIGINFOW = record
    version: ULONG;
    fConnectClientDrivesAtLogon: ULONG;
    fConnectPrinterAtLogon: ULONG;
    fDisablePrinterRedirection: ULONG;
    fDisableDefaultMainClientPrinter: ULONG;
    ShadowSettings: ULONG;
    LogonUserName: array[0..USERNAME_LENGTH] of WCHAR;
    LogonDomain: array[0..DOMAIN_LENGTH] of WCHAR;
    WorkDirectory: array[0..MAX_PATH] of WCHAR;
    InitialProgram: array[0..MAX_PATH] of WCHAR;
    ApplicationName: array[0..MAX_PATH] of WCHAR;
  end;

type
  PWTSCONFIGINFOA = ^WTSCONFIGINFOA;
  WTSCONFIGINFOA = record
    version: ULONG;
    fConnectClientDrivesAtLogon: ULONG;
    fConnectPrinterAtLogon: ULONG;
    fDisablePrinterRedirection: ULONG;
    fDisableDefaultMainClientPrinter: ULONG;
    ShadowSettings: ULONG;
    LogonUserName: array[0..USERNAME_LENGTH] of CHAR;
    LogonDomain: array[0..DOMAIN_LENGTH] of CHAR;
    WorkDirectory: array[0..MAX_PATH] of CHAR;
    InitialProgram: array[0..MAX_PATH] of CHAR;
    ApplicationName: array[0..MAX_PATH] of CHAR;
  end;

(*=====================================================================
==   WTS Session Information
=====================================================================*)
type
  PWTSINFOW = ^WTSINFOW;
  WTSINFOW = record
    State: WTS_CONNECTSTATE_CLASS;  // connection state (see enum)
    SessionId: DWORD;               // session id
    IncomingBytes: DWORD;
    OutgoingBytes: DWORD;
    IncomingFrames: DWORD;
    OutgoingFrames: DWORD;
    IncomingCompressedBytes: DWORD;
    OutgoingCompressedBytes: DWORD;
    WinStationName: array[0..(WINSTATIONNAME_LENGTH) - 1] of WCHAR;
    Domain: array[0..(DOMAIN_LENGTH - 1)] of WCHAR;
    UserName: array[0..USERNAME_LENGTH] of WCHAR;  // name of WinStation this session is
                                                   // connected to
    ConnectTime: LARGE_INTEGER;
    DisconnectTime: LARGE_INTEGER;
    LastInputTime: LARGE_INTEGER;
    LogonTime: LARGE_INTEGER;
    CurrentTime: LARGE_INTEGER;
  end;

type
  PWTSINFOA = ^WTSINFOA;
  WTSINFOA = record
    State: WTS_CONNECTSTATE_CLASS;  // connection state (see enum)
    SessionId: DWORD;               // session id
    IncomingBytes: DWORD;
    OutgoingBytes: DWORD;
    IncomingFrames: DWORD;
    OutgoingFrames: DWORD;
    IncomingCompressedBytes: DWORD;
    OutgoingCompressedBytes: DWORD;
    WinStationName: array[0..(WINSTATIONNAME_LENGTH) - 1] of CHAR;
    Domain: array[0..(DOMAIN_LENGTH - 1)] of CHAR;
    UserName: array[0..USERNAME_LENGTH] of CHAR;  // name of WinStation this session is
                                                  // connected to
    ConnectTime: LARGE_INTEGER;
    DisconnectTime: LARGE_INTEGER;
    LastInputTime: LARGE_INTEGER;
    LogonTime: LARGE_INTEGER;
    CurrentTime: LARGE_INTEGER;
  end;

(*=====================================================================
==   WTS Extended Session State Flags
=====================================================================*)
const
  WTS_SESSIONSTATE_UNKNOWN    = $FFFFFFFF;
  WTS_SESSIONSTATE_LOCK       = $00000000;
  WTS_SESSIONSTATE_UNLOCK     = $00000001;

(*=====================================================================
==   WTS Extended Session Information
=====================================================================*)
type
  PWTSINFOEX_LEVEL1_W = ^WTSINFOEX_LEVEL1_W;
  WTSINFOEX_LEVEL1_W = record
    SessionId: ULONG;
    SessionState: WTS_CONNECTSTATE_CLASS;
    SessionFlags: LONG;
    WinStationName: array[0..WINSTATIONNAME_LENGTH] of WCHAR;
    UserName: array[0..USERNAME_LENGTH] of WCHAR;
    DomainName: array[0..DOMAIN_LENGTH] of WCHAR;
    LogonTime: LARGE_INTEGER;
    ConnectTime: LARGE_INTEGER;
    DisconnectTime: LARGE_INTEGER;
    LastInputTime: LARGE_INTEGER;
    CurrentTime: LARGE_INTEGER;
    IncomingBytes: DWORD;
    OutgoingBytes: DWORD;
    IncomingFrames: DWORD;
    OutgoingFrames: DWORD;
    IncomingCompressedBytes: DWORD;
    OutgoingCompressedBytes: DWORD;
  end;

type
  PWTSINFOEX_LEVEL1_A = ^WTSINFOEX_LEVEL1_A;
  WTSINFOEX_LEVEL1_A = record
    SessionId: ULONG;
    SessionState: WTS_CONNECTSTATE_CLASS;
    SessionFlags: LONG;
    WinStationName: array[0..WINSTATIONNAME_LENGTH] of CHAR;
    UserName: array[0..USERNAME_LENGTH] of CHAR;
    DomainName: array[0..DOMAIN_LENGTH] of CHAR;
    LogonTime: LARGE_INTEGER;
    ConnectTime: LARGE_INTEGER;
    DisconnectTime: LARGE_INTEGER;
    LastInputTime: LARGE_INTEGER;
    CurrentTime: LARGE_INTEGER;
    IncomingBytes: DWORD;
    OutgoingBytes: DWORD;
    IncomingFrames: DWORD;
    OutgoingFrames: DWORD;
    IncomingCompressedBytes: DWORD;
    OutgoingCompressedBytes: DWORD;
  end;

type
  PWTSINFOEX_LEVEL_W = ^WTSINFOEX_LEVEL_W;
  WTSINFOEX_LEVEL_W = record
    WTSInfoExLevel1: WTSINFOEX_LEVEL1_W;
  end;

type
  PWTSINFOEX_LEVEL_A = ^WTSINFOEX_LEVEL_A;
  WTSINFOEX_LEVEL_A = record
    WTSInfoExLevel1: WTSINFOEX_LEVEL1_A;
  end;

type
  PWTSINFOEXW = ^WTSINFOEXW;
  WTSINFOEXW = record
    case Level: DWORD of
      1: (Data: WTSINFOEX_LEVEL_W);
  end;

type
  PWTSINFOEXA = ^WTSINFOEXA;
  WTSINFOEXA = record
    case Level: DWORD of
      1: (Data: WTSINFOEX_LEVEL_A);
  end;

(*=====================================================================
==   WTS Client Information
=====================================================================*)
type
  PWTSCLIENTW = ^WTSCLIENTW;
  WTSCLIENTW = record
    ClientName: array[0..CLIENTNAME_LENGTH] of WCHAR;
    Domain: array[0..DOMAIN_LENGTH] of WCHAR;
    UserName: array[0..USERNAME_LENGTH] of WCHAR;
    WorkDirectory: array[0..MAX_PATH] of WCHAR;
    InitialProgram: array[0..MAX_PATH] of WCHAR;
    EncryptionLevel: BYTE;      // security level of encryption pd
    ClientAddressFamily: ULONG;
    ClientAddress: array[0..CLIENTADDRESS_LENGTH] of USHORT;
    HRes: USHORT;
    VRes: USHORT;
    ColorDepth: USHORT;
    ClientDirectory: array[0..MAX_PATH] of WCHAR;
    ClientBuildNumber: ULONG;
    ClientHardwareId: ULONG;    // client software serial number
    ClientProductId: USHORT;    // client software product id
    OutBufCountHost: USHORT;    // number of outbufs on host
    OutBufCountClient: USHORT;  // number of outbufs on client
    OutBufLength: USHORT;       // length of outbufs in bytes
    DeviceId: array[0..MAX_PATH] of WCHAR;
  end;

(*=====================================================================
==   WTS Client Information
=====================================================================*)
type
  PWTSCLIENTA = ^WTSCLIENTA;
  WTSCLIENTA = record
    ClientName: array[0..CLIENTNAME_LENGTH] of CHAR;
    Domain: array[0..DOMAIN_LENGTH] of CHAR;
    UserName: array[0..USERNAME_LENGTH] of CHAR;
    WorkDirectory: array[0..MAX_PATH] of CHAR;
    InitialProgram: array[0..MAX_PATH] of CHAR;
    EncryptionLevel: BYTE;      // security level of encryption pd
    ClientAddressFamily: ULONG;
    ClientAddress: array[0..CLIENTADDRESS_LENGTH] of USHORT;
    HRes: USHORT;
    VRes: USHORT;
    ColorDepth: USHORT;
    ClientDirectory: array[0..MAX_PATH] of CHAR;
    ClientBuildNumber: ULONG;
    ClientHardwareId: ULONG;    // client software serial number
    ClientProductId: USHORT;    // client software product id
    OutBufCountHost: USHORT;    // number of outbufs on host
    OutBufCountClient: USHORT;  // number of outbufs on client
    OutBufLength: USHORT;       // length of outbufs in bytes
    DeviceId: array[0..MAX_PATH] of CHAR;
  end;

(*=====================================================================
==   WTS License Validation Information - Product Information
=====================================================================*)
const
  PRODUCTINFO_COMPANYNAME_LENGTH = 256;
  PRODUCTINFO_PRODUCTID_LENGTH = 4;

type
  PPRODUCT_INFOA = ^PRODUCT_INFOA;
  PRODUCT_INFOA = record
    CompanyName: array[0..(PRODUCTINFO_COMPANYNAME_LENGTH - 1)] of CHAR;
    ProductID: array[0..(PRODUCTINFO_PRODUCTID_LENGTH - 1)] of CHAR;
  end;

type
  PPRODUCT_INFOW = ^PRODUCT_INFOW;
  PRODUCT_INFOW = record
    CompanyName: array[0..(PRODUCTINFO_COMPANYNAME_LENGTH - 1)] of WCHAR;
    ProductID: array[0..(PRODUCTINFO_PRODUCTID_LENGTH - 1)] of WCHAR;
  end;

(*=====================================================================
     WTS License Validation Information
     This structure will be returned from WTSQuerySessionInformation when the user
     queries for license validation information.
=====================================================================*)
const
  VALIDATIONINFORMATION_LICENSE_LENGTH = 16384; //16 Kb
  VALIDATIONINFORMATION_HARDWAREID_LENGTH = 20;

type
  PWTS_VALIDATION_INFORMATIONA = ^WTS_VALIDATION_INFORMATIONA;
  WTS_VALIDATION_INFORMATIONA = record
    ProductInfo: PRODUCT_INFOA;
    License: array[0..(VALIDATIONINFORMATION_LICENSE_LENGTH -1)] of BYTE;
    LicenseLength: DWORD;
    HardwareID: array[0..(VALIDATIONINFORMATION_HARDWAREID_LENGTH -1)] of BYTE;
    HardwareIDLength: DWORD;
  end;

type
  PWTS_VALIDATION_INFORMATIONW = ^WTS_VALIDATION_INFORMATIONW;
  WTS_VALIDATION_INFORMATIONW = record
    ProductInfo: PRODUCT_INFOW;
    License: array[0..(VALIDATIONINFORMATION_LICENSE_LENGTH -1)] of BYTE;
    LicenseLength: DWORD;
    HardwareID: array[0..(VALIDATIONINFORMATION_HARDWAREID_LENGTH -1)] of BYTE;
    HardwareIDLength: DWORD;
  end;

{$IFDEF UNICODE}
type WTSCONFIGINFO = WTSCONFIGINFOW;
type PRODUCT_INFO = PRODUCT_INFOW;
type WTS_VALIDATION_INFORMATION = WTS_VALIDATION_INFORMATIONW;
type WTSINFO = WTSINFOW;
type WTSINFOEX = WTSINFOEXW;
type WTSINFOEX_LEVEL = WTSINFOEX_LEVEL_W;
type WTSINFOEX_LEVEL1 = WTSINFOEX_LEVEL1_W;
type WTSCLIENT = WTSCLIENTW;
{$ELSE}
type WTSCONFIGINFO = WTSCONFIGINFOA;
type PRODUCT_INFO = PRODUCT_INFOA;
type WTS_VALIDATION_INFORMATION = WTS_VALIDATION_INFORMATIONA;
type WTSINFO = WTSINFOA;
type WTSINFOEX = WTSINFOEXA;
type WTSINFOEX_LEVEL = WTSINFOEX_LEVEL_A;
type WTSINFOEX_LEVEL1 = WTSINFOEX_LEVEL1_A;
type WTSCLIENT = WTSCLIENTA;
{$ENDIF}

type PWTSCONFIGINFO = ^WTSCONFIGINFO;
type PPRODUCT_INFO = ^PRODUCT_INFO;
type PWTS_VALIDATION_INFORMATION = ^WTS_VALIDATION_INFORMATION;
type PWTSINFO = ^WTSINFO;
type PWTSINFOEX = ^WTSINFOEX;
type PWTSINFOEX_LEVEL = ^WTSINFOEX_LEVEL;
type PWTSINFOEX_LEVEL1 = ^WTSINFOEX_LEVEL1;
type PWTSCLIENT = ^WTSCLIENT;

(*=====================================================================
==   WTSQuerySessionInformation - (WTSClientAddress)
=====================================================================*)
type
  PWTS_CLIENT_ADDRESS = ^WTS_CLIENT_ADDRESS;
  WTS_CLIENT_ADDRESS = record
    AddressFamily: DWORD; // AF_INET, AF_INET6, AF_IPX, AF_NETBIOS, AF_UNSPEC
    Address: array[0..19] of BYTE;  // client network address
  end;

(*=====================================================================
==   WTSQuerySessionInformation - (WTSClientDisplay)
=====================================================================*)
type
  PWTS_CLIENT_DISPLAY = ^WTS_CLIENT_DISPLAY;
  WTS_CLIENT_DISPLAY = record
    HorizontalResolution: DWORD;  // horizontal dimensions, in pixels
    VerticalResolution: DWORD;    // vertical dimensions, in pixels
    ColorDepth: DWORD;           // 1=16, 2=256, 4=64K, 8=16M
  end;

(*=====================================================================
==   WTS_CONFIG_CLASS - WTSQueryUserConfig/WTSSetUserConfig
=====================================================================*)
type WTS_CONFIG_CLASS = (
  //Initial program settings
  WTSUserConfigInitialProgram,            // string returned/expected
  WTSUserConfigWorkingDirectory,          // string returned/expected
  WTSUserConfigfInheritInitialProgram,    // DWORD returned/expected
  //
  WTSUserConfigfAllowLogonTerminalServer,     //DWORD returned/expected
  //Timeout settings
  WTSUserConfigTimeoutSettingsConnections,    //DWORD returned/expected
  WTSUserConfigTimeoutSettingsDisconnections, //DWORD returned/expected
  WTSUserConfigTimeoutSettingsIdle,           //DWORD returned/expected
  //Client device settings
  WTSUserConfigfDeviceClientDrives,       //DWORD returned/expected
  WTSUserConfigfDeviceClientPrinters,         //DWORD returned/expected
  WTSUserConfigfDeviceClientDefaultPrinter,   //DWORD returned/expected
  //Connection settings
  WTSUserConfigBrokenTimeoutSettings,         //DWORD returned/expected
  WTSUserConfigReconnectSettings,             //DWORD returned/expected
  //Modem settings
  WTSUserConfigModemCallbackSettings,         //DWORD returned/expected
  WTSUserConfigModemCallbackPhoneNumber,      // string returned/expected
  //Shadow settings
  WTSUserConfigShadowingSettings,             //DWORD returned/expected
  //User Profile settings
  WTSUserConfigTerminalServerProfilePath,     // string returned/expected
  //Terminal Server home directory
  WTSUserConfigTerminalServerHomeDir,       // string returned/expected
  WTSUserConfigTerminalServerHomeDirDrive,    // string returned/expected
  WTSUserConfigfTerminalServerRemoteHomeDir,  // DWORD 0:LOCAL 1:REMOTE

  WTSUserConfigUser                           // returns WTSUSERCONFIG struct
);

type WTS_CONFIG_SOURCE = (
  WTSUserConfigSourceSAM
);

type
  PWTSUSERCONFIGA = ^WTSUSERCONFIGA;
  WTSUSERCONFIGA = record
    Source: DWORD;
    InheritInitialProgram: DWORD;
    AllowLogonTerminalServer: DWORD;
    TimeoutSettingsConnections: DWORD;
    TimeoutSettingsDisconnections: DWORD;
    TimeoutSettingsIdle: DWORD;
    DeviceClientDrives: DWORD;
    DeviceClientPrinters: DWORD;
    ClientDefaultPrinter: DWORD;
    BrokenTimeoutSettings: DWORD;
    ReconnectSettings: DWORD;
    ShadowingSettings: DWORD;
    TerminalServerRemoteHomeDir: DWORD;
    InitialProgram: array[0..MAX_PATH] of CHAR;
    WorkDirectory: array[0..MAX_PATH] of CHAR;
    TerminalServerProfilePath: array[0..MAX_PATH] of CHAR;
    TerminalServerHomeDir: array[0..MAX_PATH] of CHAR;
    TerminalServerHomeDirDrive: array[0..WTS_DRIVE_LENGTH] of CHAR;
  end;

type
  PWTSUSERCONFIGW = ^WTSUSERCONFIGW;
  WTSUSERCONFIGW = record
    Source: DWORD;
    InheritInitialProgram: DWORD;
    AllowLogonTerminalServer: DWORD;
    TimeoutSettingsConnections: DWORD;
    TimeoutSettingsDisconnections: DWORD;
    TimeoutSettingsIdle: DWORD;
    DeviceClientDrives: DWORD;
    DeviceClientPrinters: DWORD;
    ClientDefaultPrinter: DWORD;
    BrokenTimeoutSettings: DWORD;
    ReconnectSettings: DWORD;
    ShadowingSettings: DWORD;
    TerminalServerRemoteHomeDir: DWORD;
    InitialProgram: array[0..MAX_PATH] of WCHAR;
    WorkDirectory: array[0..MAX_PATH] of WCHAR;
    TerminalServerProfilePath: array[0..MAX_PATH] of WCHAR;
    TerminalServerHomeDir: array[0..MAX_PATH] of WCHAR;
    TerminalServerHomeDirDrive: array[0..WTS_DRIVE_LENGTH] of WCHAR;
  end;

{$IFDEF UNICODE}
type WTSUSERCONFIG = WTSUSERCONFIGW;
{$ELSE}
type WTSUSERCONFIG = WTSUSERCONFIGA;
{$ENDIF}

type PWTSUSERCONFIG = ^WTSUSERCONFIG;

(*=====================================================================
==   WTS_EVENT - Event flags for WTSWaitSystemEvent
=====================================================================*)
const
   WTS_EVENT_NONE         = $00000000; // return no event
   WTS_EVENT_CREATE       = $00000001; // new WinStation created
   WTS_EVENT_DELETE       = $00000002; // existing WinStation deleted
   WTS_EVENT_RENAME       = $00000004; // existing WinStation renamed
   WTS_EVENT_CONNECT      = $00000008; // WinStation connect to client
   WTS_EVENT_DISCONNECT   = $00000010; // WinStation logged on without
                                       //     client
   WTS_EVENT_LOGON        = $00000020; // user logged on to existing
                                       //     WinStation
   WTS_EVENT_LOGOFF       = $00000040; // user logged off from
                                       //     existing WinStation
   WTS_EVENT_STATECHANGE  = $00000080; // WinStation state change
   WTS_EVENT_LICENSE      = $00000100; // license state change
   WTS_EVENT_ALL          = $7fffffff; // wait for all event types
   WTS_EVENT_FLUSH        = $80000000; // unblock all waiters

(*=====================================================================
==   Flags for HotkeyModifiers in WTSStartRemoteControlSession
=====================================================================*)
const
   REMOTECONTROL_KBDSHIFT_HOTKEY              = $1;    // Shift key
   REMOTECONTROL_KBDCTRL_HOTKEY               = $2;    // Ctrl key
   REMOTECONTROL_KBDALT_HOTKEY                = $4;    // Alt key

(*=====================================================================
==   WTS_VIRTUAL_CLASS - WTSVirtualChannelQuery
=====================================================================*)
type WTS_VIRTUAL_CLASS = (
  WTSVirtualClientData,  // Virtual channel client module data
                         //     (C2H data)
  WTSVirtualFileHandle
);

(*=====================================================================
==   WTSQuerySessionInformation - (WTSSessionAddress)
=====================================================================*)
type
  PWTS_SESSION_ADDRESS = ^WTS_SESSION_ADDRESS;
  WTS_SESSION_ADDRESS = record
    AddressFamily: DWORD;           // AF_INET only.
    Address: array[0..19] of BYTE;  // client network address
  end;

(*=====================================================================
==   Windows Terminal Server public APIs
=====================================================================*)
function WTSStopRemoteControlSession(LogonID: ULONG): WINBOOL; stdcall;
  external wtsapi32_dll;

function WTSStartRemoteControlSessionW(pTargetServerName: LPWSTR;
  TargetLogonId: ULONG; HotkeyVk: BYTE; HotkeyModifiers: USHORT): WINBOOL;
  stdcall; external wtsapi32_dll;

function WTSStartRemoteControlSessionA(pTargetServerName: LPSTR;
  TargetLogonId: ULONG; HotkeyVk: BYTE; HotkeyModifiers: USHORT): WINBOOL;
  stdcall; external wtsapi32_dll;

{$IFDEF UNICODE}
function WTSStartRemoteControlSession(pTargetServerName: LPWSTR;
  TargetLogonId: ULONG; HotkeyVk: BYTE; HotkeyModifiers: USHORT): WINBOOL;
  stdcall; external wtsapi32_dll name 'WTSStartRemoteControlSessionW';
{$ELSE}
function WTSStartRemoteControlSession(pTargetServerName: LPSTR;
  TargetLogonId: ULONG; HotkeyVk: BYTE; HotkeyModifiers: USHORT): WINBOOL;
  stdcall; external wtsapi32_dll name 'WTSStartRemoteControlSessionA';
{$ENDIF}

function WTSConnectSessionA(LogonId, TargetLogonId: ULONG;
  pPassword: PSTR; bWait: WINBOOL): WINBOOL; stdcall; external wtsapi32_dll;

function WTSConnectSessionW(LogonId, TargetLogonId: ULONG;
  pPassword: PWSTR; bWait: WINBOOL): WINBOOL; stdcall; external wtsapi32_dll;

{$IFDEF UNICODE}
function WTSConnectSession(LogonId, TargetLogonId: ULONG;
  pPassword: PWSTR; bWait: WINBOOL): WINBOOL; stdcall;
  external wtsapi32_dll name 'WTSConnectSessionW';
{$ELSE}
function WTSConnectSession(LogonId, TargetLogonId: ULONG;
  pPassword: PSTR; bWait: WINBOOL): WINBOOL; stdcall;
  external wtsapi32_dll name 'WTSConnectionSessionA';
{$ENDIF}

function WTSEnumerateServersW(pDomainName: LPWSTR;
  Reserved, Version: DWORD; ppServerInfo: PPWTS_SERVER_INFOW;
  pCount: PDWORD): WINBOOL; stdcall; external wtsapi32_dll;

function WTSEnumerateServersA(pDomainName: LPSTR;
  Reserved, Version: DWORD; ppServerInfo: PPWTS_SERVER_INFOA;
  pCount: PDWORD): WINBOOL; stdcall; external wtsapi32_dll;

{$IFDEF UNICODE}
function WTSEnumerateServers(pDomainName: LPWSTR;
  Reserved, Version: DWORD; ppServerInfo: PPWTS_SERVER_INFOW;
  pCount: PDWORD): WINBOOL; stdcall;
  external wtsapi32_dll name 'WTSEnumerateServersW';
{$ELSE}
function WTSEnumerateServers(pDomainName: LPSTR;
  Reserved, Version: DWORD; ppServerInfo: PPWTS_SERVER_INFOA;
  pCount: PDWORD): WINBOOL; stdcall;
  external wtsapi32_dll name 'WTSEnumerateServersA';
{$ENDIF}

(*------------------------------------------------*)

function WTSOpenServerW(pServerName: LPWSTR): HANDLE; stdcall;
  external wtsapi32_dll;

function WTSOpenServerA(pServerName: LPSTR): HANDLE; stdcall;
  external wtsapi32_dll;

function WTSOpenServerExW(pServerName: LPWSTR): HANDLE; stdcall;
  external wtsapi32_dll;

function WTSOpenServerExA(pServerName: LPSTR): HANDLE; stdcall;
  external wtsapi32_dll;

{$IFDEF UNICODE}
function WTSOpenServer(pServerName: LPWSTR): HANDLE; stdcall;
  external wtsapi32_dll name 'WTSOpenServerW';
  function WTSOpenServerEx(pServerName: LPWSTR): HANDLE; stdcall;
  external wtsapi32_dll name 'WTSOpenServerExW';
{$ELSE}
function WTSOpenServer(pServerName: LPSTR): HANDLE; stdcall;
  external wtsapi32_dll name 'WTSOpenServerA';
  function WTSOpenServerEx(pServerName: LPSTR): HANDLE; stdcall;
  external wtsapi32_dll name 'WTSOpenServerExA';
{$ENDIF}

(*------------------------------------------------*)

procedure WTSCloseServer(hServer: HANDLE); stdcall; external wtsapi32_dll;

(*------------------------------------------------*)

function WTSEnumerateSessionsW(hServer: HANDLE; Reserved, Version: DWORD;
  ppSessionInfo: PPWTS_SESSION_INFOW; pCount: PDWORD): WINBOOL; stdcall;
  external wtsapi32_dll;

function WTSEnumerateSessionsA(hServer: HANDLE; Reserved, Version: DWORD;
  ppSessionInfo: PPWTS_SESSION_INFOA; pCount: PDWORD): WINBOOL; stdcall;
  external wtsapi32_dll;

function WTSEnumerateSessionsExW(hServer: HANDLE; pLevel: PDWORD;
  Filter: DWORD; ppSessionInfo: PPWTS_SESSION_INFO_1W;
  pCount: PDWORD): WINBOOL; stdcall; external wtsapi32_dll;

function WTSEnumerateSessionsExA(hServer: HANDLE; pLevel: PDWORD;
  Filter: DWORD; ppSessionInfo: PPWTS_SESSION_INFO_1A;
  pCount: PDWORD): WINBOOL; stdcall; external wtsapi32_dll;

{$IFDEF UNICODE}
function WTSEnumerateSessions(hServer: HANDLE; Reserved, Version: DWORD;
  ppSessionInfo: PPWTS_SESSION_INFOW; pCount: PDWORD): WINBOOL; stdcall;
  external wtsapi32_dll name 'WTSEnumerateSessionsW';

function WTSEnumerateSessionsEx(hServer: HANDLE; pLevel: PDWORD;
  Filter: DWORD; ppSessionInfo: PPWTS_SESSION_INFO_1W;
  pCount: PDWORD): WINBOOL; stdcall;
  external wtsapi32_dll name 'WTSEnumerateSessionsExW';

{$ELSE}
function WTSEnumerateSessions(hServer: HANDLE; Reserved, Version: DWORD;
  ppSessionInfo: PPWTS_SESSION_INFOA; pCount: PDWORD): WINBOOL; stdcall;
  external wtsapi32_dll name 'WTSEnumerateSessionsA';

function WTSEnumerateSessionsEx(hServer: HANDLE; pLevel: PDWORD;
  Filter: DWORD; ppSessionInfo: PPWTS_SESSION_INFO_1A;
  pCount: PDWORD): WINBOOL; stdcall;
  external wtsapi32_dll name 'WTSEnumerateSessionsExA';
{$ENDIF}

(*------------------------------------------------*)

function WTSEnumerateProcessesW(hServer: HANDLE; Reserved, Version: DWORD;
  ppProcessInfo: PPWTS_PROCESS_INFOW; pCount: PDWORD): WINBOOL; stdcall;
  external wtsapi32_dll;

function WTSEnumerateProcessesA(hServer: HANDLE; Reserved, Version: DWORD;
  ppProcessInfo: PPWTS_PROCESS_INFOA; pCount: PDWORD): WINBOOL; stdcall;
  external wtsapi32_dll;

{$IFDEF UNICODE}
function WTSEnumerateProcesses(hServer: HANDLE; Reserved, Version: DWORD;
  ppProcessInfo: PPWTS_PROCESS_INFOW; pCount: PDWORD): WINBOOL; stdcall;
  external wtsapi32_dll name 'WTSEnumerateProcessesW';
{$ELSE}
function WTSEnumerateProcesses(hServer: HANDLE; Reserved, Version: DWORD;
  ppProcessInfo: PPWTS_PROCESS_INFOA; pCount: PDWORD): WINBOOL; stdcall;
  external wtsapi32_dll name 'WTSEnumerateProcessesA';
{$ENDIF}

(*------------------------------------------------*)

function WTSTerminateProcess(hServer: HANDLE; ProcessId: DWORD;
  ExitCode: DWORD): WINBOOL; stdcall; external wtsapi32_dll;

(*------------------------------------------------*)

function WTSQuerySessionInformationW(hServer: HANDLE;
  SessionID: DWORD; WTSInfoClass: WTS_INFO_CLASS;
  ppBuffer: LPWSTR; pBytesReturned: PDWORD): WINBOOL; stdcall;
  external wtsapi32_dll;

function WTSQuerySessionInformationA(hServer: HANDLE;
  SessionID: DWORD; WTSInfoClass: WTS_INFO_CLASS;
  ppBuffer: LPSTR; pBytesReturned: PDWORD): WINBOOL; stdcall;
  external wtsapi32_dll;

{$IFDEF UNICODE}
function WTSQuerySessionInformation(hServer: HANDLE;
  SessionID: DWORD; WTSInfoClass: WTS_INFO_CLASS;
  ppBuffer: PLPWSTR; pBytesReturned: PDWORD): WINBOOL; stdcall;
  external wtsapi32_dll name 'WTSQuerySessionInformationW';
{$ELSE}
function WTSQuerySessionInformation(hServer: HANDLE;
  SessionID: DWORD; WTSInfoClass: WTS_INFO_CLASS;
  ppBuffer: PLPSTR; pBytesReturned: PDWORD): WINBOOL; stdcall;
  external wtsapi32_dll name 'WTSQuerySessionInformationA';
{$ENDIF}

(*------------------------------------------------*)

function WTSQueryUserConfigW(pServerName, pUserName: LPWSTR;
  WTSConfigClass: WTS_CONFIG_CLASS; ppBuffer: PLPWSTR;
  pBytesReturned: PDWORD): WINBOOL; stdcall; external wtsapi32_dll;

function WTSQueryUserConfigA(pServerName, pUserName: LPSTR;
  WTSConfigClass: WTS_CONFIG_CLASS; ppBuffer: PLPSTR;
  pBytesReturned: PDWORD): WINBOOL; stdcall; external wtsapi32_dll;

{$IFDEF UNICODE}
function WTSQueryUserConfig(pServerName, pUserName: LPWSTR;
  WTSConfigClass: WTS_CONFIG_CLASS; ppBuffer: PLPWSTR;
  pBytesReturned: PDWORD): WINBOOL; stdcall;
  external wtsapi32_dll name 'WTSQueryUserConfigW';
{$ELSE}
function WTSQueryUserConfig(pServerName, pUserName: LPSTR;
  WTSConfigClass: WTS_CONFIG_CLASS; ppBuffer: PLPSTR;
  pBytesReturned: PDWORD): WINBOOL; stdcall;
  external wtsapi32_dll name 'WTSQueryUserConfigA';
{$ENDIF}

(*------------------------------------------------*)

function WTSSetUserConfigW(pServerName, pUserName: LPWSTR;
  WTSConfigClass: WTS_CONFIG_CLASS; pBuffer: LPWSTR;
  DataLength: DWORD): WINBOOL; stdcall; external wtsapi32_dll;

function WTSSetUserConfigA(pServerName, pUserName: LPSTR;
  WTSConfigClass: WTS_CONFIG_CLASS; pBuffer: LPSTR;
  DataLEngth: DWORD): WINBOOL; stdcall; external wtsapi32_dll;

{$IFDEF UNICODE}
function WTSSetUserConfig(pServerName, pUserName: LPWSTR;
  WTSConfigClass: WTS_CONFIG_CLASS; pBuffer: LPWSTR;
  DataLength: DWORD): WINBOOL; stdcall;
  external wtsapi32_dll name 'WTSSetUserConfigW';
{$ELSE}
function WTSSetUserConfig(pServerName, pUserName: LPSTR;
  WTSConfigClass: WTS_CONFIG_CLASS; pBuffer: LPSTR;
  DataLEngth: DWORD): WINBOOL; stdcall;
  external wtsapi32_dll name 'WTSSetUserConfigA';
{$ENDIF}

(*------------------------------------------------*)

function WTSSendMessageW(hServer: HANDLE; SessionId: DWORD;
  pTitle: LPWSTR; TitleLength: DWORD;
  pMessage: LPWSTR; MessageLength: DWORD; Style, Timeout: DWORD;
  pResponse: PDWORD; bWait: WINBOOL): WINBOOL; stdcall;
  external wtsapi32_dll;

function WTSSendMessageA(hServer: HANDLE; SessionId: DWORD;
  pTitle: LPSTR; TitleLength: DWORD;
  pMessage: LPSTR; MessageLength: DWORD; Style, Timeout: DWORD;
  pResponse: PDWORD; bWait: WINBOOL): WINBOOL; stdcall;
  external wtsapi32_dll;

{$IFDEF UNICODE}
function WTSSendMessage(hServer: HANDLE; SessionId: DWORD;
  pTitle: LPWSTR; TitleLength: DWORD;
  pMessage: LPWSTR; MessageLength: DWORD; Style, Timeout: DWORD;
  pResponse: PDWORD; bWait: WINBOOL): WINBOOL; stdcall;
  external wtsapi32_dll name 'WTSSendMessageW';
{$ELSE}
function WTSSendMessage(hServer: HANDLE; SessionId: DWORD;
  pTitle: LPSTR; TitleLength: DWORD;
  pMessage: LPSTR; MessageLength: DWORD; Style, Timeout: DWORD;
  pResponse: PDWORD; bWait: WINBOOL): WINBOOL; stdcall;
  external wtsapi32_dll name 'WTSSendMessageA';
{$ENDIF}

(*------------------------------------------------*)

function WTSDisconnectSession(hServer: HANDLE; SessionId: DWORD;
  bWait: WINBOOL): WINBOOL; stdcall; external wtsapi32_dll;

(*------------------------------------------------*)

function WTSLogoffSession(hServer: HANDLE; SessionId: DWORD;
  bWait: WINBOOL): WINBOOL; stdcall; external wtsapi32_dll;

(*------------------------------------------------*)

function WTSShutdownSystem(hServer: HANDLE; ShutdownFlag: DWORD): WINBOOL;
  stdcall; external wtsapi32_dll;

(*------------------------------------------------*)

function WTSWaitSystemEvent(hServer: HANDLE; EventMask: DWORD;
  pEventFlags: PDWORD): WINBOOL; stdcall; external wtsapi32_dll;

(*------------------------------------------------*)

function WTSVirtualChannelOpen(hServer: HANDLE; SessionId: DWORD;
  // ascii name
  pVirtualName: LPSTR): HANDLE; stdcall; external wtsapi32_dll;

const
   WTS_CHANNEL_OPTION_DYNAMIC          = $00000001;   // dynamic channel
   WTS_CHANNEL_OPTION_DYNAMIC_PRI_LOW  = $00000000;   // priorities
   WTS_CHANNEL_OPTION_DYNAMIC_PRI_MED  = $00000002;
   WTS_CHANNEL_OPTION_DYNAMIC_PRI_HIGH = $00000004;
   WTS_CHANNEL_OPTION_DYNAMIC_PRI_REAL = $00000006;
   WTS_CHANNEL_OPTION_DYNAMIC_NO_COMPRESS = $00000008;

function WTSVirtualChannelOpenEx(SessionId: DWORD;
  // ascii name
  pVirtualName: LPSTR;
  flags: DWORD): HANDLE; stdcall; external wtsapi32_dll;

function WTSVirtualChannelClose(hChannelHandle: HANDLE): WINBOOL; stdcall;
  external wtsapi32_dll;

function WTSVirtualChannelRead(hChannelHandle: HANDLE; TimeOut: ULONG;
  Buffer: PCHAR; BufferSize: ULONG; pBytesRead: PULONG): WINBOOL; stdcall;
  external wtsapi32_dll;

function WTSVirtualChannelWrite(hChannelHandle: HANDLE; Buffer: PCHAR;
  Length: ULONG; pBytesWritten: PULONG): WINBOOL; stdcall;
  external wtsapi32_dll;

function WTSVirtualChannelPurgeInput(hChannelHandle: HANDLE): WINBOOL; stdcall;
  external wtsapi32_dll;

function WTSVirtualChannelPurgeOutput(hChannelHandle: HANDLE): WINBOOL; stdcall;
  external wtsapi32_dll;

function WTSVirtualChannelQuery(hChannelHandle: HANDLE;
  WTSVirtualClass: WTS_VIRTUAL_CLASS; ppBuffer: PPVOID;
  pBytesReturned: PDWORD): WINBOOL; stdcall; external wtsapi32_dll;

(*------------------------------------------------*)

procedure WTSFreeMemory(pMemory: PVOID); stdcall; external wtsapi32_dll;

(* Flags for Console Notification *)
const
  NOTIFY_FOR_ALL_SESSIONS = 1;
  NOTIFY_FOR_THIS_SESSION = 0;

function WTSRegisterSessionNotification(_hWnd: HWND; dwFlags: DWORD): WINBOOL;
  stdcall; external wtsapi32_dll;

function WTSUnRegisterSessionNotification(_hwnd: HWND): WINBOOL; stdcall;
  external wtsapi32_dll;

function WTSRegisterSessionNotificationEx(hServer: HANDLE;
  _hWnd: HWND; dwFlags: DWORD): WINBOOL; stdcall; external wtsapi32_dll;

function WTSUnRegisterSessionNotificationEx(hServer: HANDLE;
  _hWnd: HWND): WINBOOL; stdcall; external wtsapi32_dll;

function WTSQueryUserToken(SessionId: ULONG; phToken: PHANDLE): WINBOOL;
  stdcall; external wtsapi32_dll;

const
  WTS_PROCESS_INFO_LEVEL_0 = 0;
  WTS_PROCESS_INFO_LEVEL_1 = 1;

(*
==   WTS_PROCESS_INFO_EX - returned by WTSEnumerateProcessesEX
*)
type
  PWTS_PROCESS_INFO_EXW = ^WTS_PROCESS_INFO_EXW;
  WTS_PROCESS_INFO_EXW = record
    SessionId: DWORD;
    ProcessId: DWORD;
    pProcessName: LPWSTR;
    pUserSid: PSID;
    NumberOfThreads: DWORD;
    HandleCount: DWORD;
    PagefileUsage: DWORD;
    PeakPagefileUsage: DWORD;
    WorkingSetSize: DWORD;
    PeakWorkingSetSize: DWORD;
    UserTime: LARGE_INTEGER;
    KernelTime: LARGE_INTEGER;
  end;

type
  PWTS_PROCESS_INFO_EXA = ^WTS_PROCESS_INFO_EXA;
  WTS_PROCESS_INFO_EXA = record
    SessionId: DWORD;
    ProcessId: DWORD;
    pProcessName: LPSTR;
    pUserSid: PSID;
    NumberOfThreads: DWORD;
    HandleCount: DWORD;
    PagefileUsage: DWORD;
    PeakPagefileUsage: DWORD;
    WorkingSetSize: DWORD;
    PeakWorkingSetSize: DWORD;
    UserTime: LARGE_INTEGER;
    KernelTime: LARGE_INTEGER;
  end;

{$IFDEF UNICODE}
type WTS_PROCESS_INFO_EX = WTS_PROCESS_INFO_EXW;
{$ELSE}
type WTS_PROCESS_INFO_EX = WTS_PROCESS_INFO_EXA;
{$ENDIF}

type PWTS_PROCESS_INFO_EX = ^WTS_PROCESS_INFO_EX;

(*------------------------------------------------*)

type WTS_TYPE_CLASS = (
  WTSTypeProcessInfoLevel0,
  WTSTypeProcessInfoLevel1,
  WTSTypeSessionInfoLevel1
);

function WTSFreeMemoryExW(WTSTypeClass: WTS_TYPE_CLASS;
  pMemory: PVOID; NumberOfEntries: ULONG): WINBOOL; stdcall;
  external wtsapi32_dll;

function WTSFreeMemoryExA(WTSTypeClass: WTS_TYPE_CLASS;
  pMemory: PVOID; NumberOfEntries: ULONG): WINBOOL; stdcall;
  external wtsapi32_dll;

{$IFDEF UNICODE}
function WTSFreeMemoryEx(WTSTypeClass: WTS_TYPE_CLASS;
  pMemory: PVOID; NumberOfEntries: ULONG): WINBOOL; stdcall;
  external wtsapi32_dll name 'WTSFreeMemoryExW';
{$ELSE}
function WTSFreeMemoryEx(WTSTypeClass: WTS_TYPE_CLASS;
  pMemory: PVOID; NumberOfEntries: ULONG): WINBOOL; stdcall;
  external wtsapi32_dll name 'WTSFreeMemoryExA';
{$ENDIF}

(*------------------------------------------------*)

function WTSEnumerateProcessesExW(hServer: HANDLE;
  pLevel: PDWORD; SessionId: DWORD; ppProcessInfo: PLPWSTR;
  pCount: PDWORD): WINBOOL; stdcall; external wtsapi32_dll;

function WTSEnumerateProcessesExA(hServer: HANDLE;
  pLevel: PDWORD; SessionId: DWORD; ppProcessInfo: PLPSTR;
  pCount: PDWORD): WINBOOL; stdcall; external wtsapi32_dll;

{$IFDEF UNICODE}
function WTSEnumerateProcessesEx(hServer: HANDLE;
  pLevel: PDWORD; SessionId: DWORD; ppProcessInfo: PLPWSTR;
  pCount: PDWORD): WINBOOL; stdcall;
  external wtsapi32_dll name 'WTSEnumerateProcessesExW';
{$ELSE}
function WTSEnumerateProcessesEx(hServer: HANDLE;
  pLevel: PDWORD; SessionId: DWORD; ppProcessInfo: PLPSTR;
  pCount: PDWORD): WINBOOL; stdcall;
  external wtsapi32_dll name 'WTSEnumerateProcessesExA';
{$ENDIF}

(*------------------------------------------------*)
// Listener management APIs
type
  PWTSLISTENERNAMEW = ^WTSLISTENERNAMEW;
  WTSLISTENERNAMEW = array[0..WTS_LISTENER_NAME_LENGTH] of WCHAR;

type
  PWTSLISTENERNAMEA = ^WTSLISTENERNAMEA;
  WTSLISTENERNAMEA = array[0..WTS_LISTENER_NAME_LENGTH] of CHAR;

{$IFDEF UNICODE}
type WTSLISTENERNAME = WTSLISTENERNAMEW;
{$ELSE}
type WTSLISTENERNAME = WTSLISTENERNAMEA;
{$ENDIF}

type PWTSLISTENERNAME = ^WTSLISTENERNAME;

function WTSEnumerateListenersW(hServer: HANDLE; pReserved: PVOID;
  Reserved: DWORD; pListeners: PWTSLISTENERNAMEW; pCount: PDWORD): WINBOOL;
  stdcall; external wtsapi32_dll;

function WTSEnumerateListenersA(hServer: HANDLE; pReserved: PVOID;
  Reserved: DWORD; pListeners: PWTSLISTENERNAMEA; pCount: PDWORD): WINBOOL;
  stdcall; external wtsapi32_dll;

{$IFDEF UNICODE}
function WTSEnumerateListeners(hServer: HANDLE; pReserved: PVOID;
  Reserved: DWORD; pListeners: PWTSLISTENERNAMEW; pCount: PDWORD): WINBOOL;
  stdcall; external wtsapi32_dll name 'WTSEnumerateListenersW';
{$ELSE}
function WTSEnumerateListeners(hServer: HANDLE; pReserved: PVOID;
  Reserved: DWORD; pListeners: PWTSLISTENERNAMEA; pCount: PDWORD): WINBOOL;
  stdcall; external wtsapi32_dll name 'WTSEnumerateListenersA';
{$ENDIF}

(*------------------------------------------------*)

// Listener Config, used by WTSQueryListenerConfig and WTSCreateListener
type
  PWTSLISTENERCONFIGW = ^WTSLISTENERCONFIGW;
  WTSLISTENERCONFIGW = record
    version: ULONG;
    fEnableListener: ULONG;
    MaxConnectionCount: ULONG;
    fPromptForPassword: ULONG;
    fInheritColorDepth: ULONG;
    ColorDepth: ULONG;
    fInheritBrokenTimeoutSettings: ULONG;
    BrokenTimeoutSettings: ULONG;

    fDisablePrinterRedirection: ULONG;
    fDisableDriveRedirection: ULONG;
    fDisableComPortRedirection: ULONG;
    fDisableLPTPortRedirection: ULONG;
    fDisableClipboardRedirection: ULONG;
    fDisableAudioRedirection: ULONG;
    fDisablePNPRedirection: ULONG;
    fDisableDefaultMainClientPrinter: ULONG;

    LanAdapter: ULONG;
    PortNumber: ULONG;

    fInheritShadowSettings: ULONG;
    ShadowSettings: ULONG;

    TimeoutSettingsConnection: ULONG;
    TimeoutSettingsDisconnection: ULONG;
    TimeoutSettingsIdle: ULONG;

    SecurityLayer: ULONG;
    MinEncryptionLevel: ULONG;
    UserAuthentication: ULONG;

    Comment: array[0..WTS_COMMENT_LENGTH] of WCHAR;
    LogonUserName: array[0..USERNAME_LENGTH] of WCHAR;
    LogonDomain: array[0..DOMAIN_LENGTH] of WCHAR;

    WorkDirectory: array[0..MAX_PATH] of WCHAR;
    InitialProgram: array[0..MAX_PATH] of WCHAR;
  end;

type
  PWTSLISTENERCONFIGA = ^WTSLISTENERCONFIGA;
  WTSLISTENERCONFIGA = record
    version: ULONG;
    fEnableListener: ULONG;
    MaxConnectionCount: ULONG;
    fPromptForPassword: ULONG;
    fInheritColorDepth: ULONG;
    ColorDepth: ULONG;
    fInheritBrokenTimeoutSettings: ULONG;
    BrokenTimeoutSettings: ULONG;

    fDisablePrinterRedirection: ULONG;
    fDisableDriveRedirection: ULONG;
    fDisableComPortRedirection: ULONG;
    fDisableLPTPortRedirection: ULONG;
    fDisableClipboardRedirection: ULONG;
    fDisableAudioRedirection: ULONG;
    fDisablePNPRedirection: ULONG;
    fDisableDefaultMainClientPrinter: ULONG;

    LanAdapter: ULONG;
    PortNumber: ULONG;

    fInheritShadowSettings: ULONG;
    ShadowSettings: ULONG;

    TimeoutSettingsConnection: ULONG;
    TimeoutSettingsDisconnection: ULONG;
    TimeoutSettingsIdle: ULONG;

    SecurityLayer: ULONG;
    MinEncryptionLevel: ULONG;
    UserAuthentication: ULONG;

    Comment: array[0..WTS_COMMENT_LENGTH] of CHAR;
    LogonUserName: array[0..USERNAME_LENGTH] of CHAR;
    LogonDomain: array[0..DOMAIN_LENGTH] of CHAR;

    WorkDirectory: array[0..MAX_PATH] of CHAR;
    InitialProgram: array[0..MAX_PATH] of CHAR;
  end;

function WTSQueryListenerConfigW(hServer: HANDLE; pReserved: PVOID;
  Reserved: DWORD; pListenerName: LPWSTR;
  pBuffer: PWTSLISTENERCONFIGW): WINBOOL; stdcall; external wtsapi32_dll;

function WTSQueryListenerConfigA(hServer: HANDLE; pReserved: PVOID;
  Reserved: DWORD; pListenerName: LPSTR;
  pBuffer: PWTSLISTENERCONFIGA): WINBOOL; stdcall; external wtsapi32_dll;

function WTSCreateListenerW(hServer: HANDLE; pReserved: PVOID;
  Reserved: DWORD; pListenerName: LPWSTR; pBuffer: PWTSLISTENERCONFIGW;
  flag: DWORD): WINBOOL; stdcall; external wtsapi32_dll;

function WTSCreateListenerA(hServer: HANDLE; pReserved: PVOID;
  Reserved: DWORD; pListenerName: LPSTR; pBuffer: PWTSLISTENERCONFIGA;
  flag: DWORD): WINBOOL; stdcall; external wtsapi32_dll;

function WTSSetListenerSecurityW(hServer: HANDLE; pReserved: PVOID;
  Reserved: DWORD; pListenerName: LPWSTR;
  SecurityInformation: SECURITY_INFORMATION;
  pSecurityDescriptor: PSECURITY_DESCRIPTOR): WINBOOL; stdcall;
  external wtsapi32_dll;

function WTSSetListenerSecurityA(hServer: HANDLE; pReserved: PVOID;
  Reserved: DWORD; pListenerName: LPSTR;
  SecurityInformation: SECURITY_INFORMATION;
  pSecurityDescriptor: PSECURITY_DESCRIPTOR): WINBOOL; stdcall;
  external wtsapi32_dll;

function WTSGetListenerSecurityW(hServer: HANDLE; pReserved: PVOID;
  Reserved: DWORD; pListenerName: LPWSTR;
  SecurityInformation: SECURITY_INFORMATION;
  pSecurityDescriptor: PSECURITY_DESCRIPTOR; nLength: DWORD;
  lpnLengthNeeded: LPDWORD): WINBOOL; stdcall; external wtsapi32_dll;

function WTSGetListenerSecurityA(hServer: HANDLE; pReserved: PVOID;
  Reserved: DWORD; pListenerName: LPSTR;
  SecurityInformation: SECURITY_INFORMATION;
  pSecurityDescriptor: PSECURITY_DESCRIPTOR; nLength: DWORD;
  lpnLengthNeeded: LPDWORD): WINBOOL; stdcall; external wtsapi32_dll;

{$IFDEF UNICODE}
type WTSLISTENERCONFIG = WTSLISTENERCONFIGW;

function WTSQueryListenerConfig(hServer: HANDLE; pReserved: PVOID;
  Reserved: DWORD; pListenerName: LPWSTR;
  pBuffer: PWTSLISTENERCONFIGW): WINBOOL; stdcall;
  external wtsapi32_dll name 'WTSQueryListenerConfigW';

function WTSCreateListener(hServer: HANDLE; pReserved: PVOID;
  Reserved: DWORD; pListenerName: LPWSTR; pBuffer: PWTSLISTENERCONFIGW;
  flag: DWORD): WINBOOL; stdcall;
  external wtsapi32_dll name 'WTSCreateListenerW';

function WTSSetListenerSecurity(hServer: HANDLE; pReserved: PVOID;
  Reserved: DWORD; pListenerName: LPWSTR;
  SecurityInformation: SECURITY_INFORMATION;
  pSecurityDescriptor: PSECURITY_DESCRIPTOR): WINBOOL; stdcall;
  external wtsapi32_dll name 'WTSSetListenerSecurityW';

function WTSGetListenerSecurity(hServer: HANDLE; pReserved: PVOID;
  Reserved: DWORD; pListenerName: LPWSTR;
  SecurityInformation: SECURITY_INFORMATION;
  pSecurityDescriptor: PSECURITY_DESCRIPTOR; nLength: DWORD;
  lpnLengthNeeded: LPDWORD): WINBOOL; stdcall;
  external wtsapi32_dll name 'WTSGetListenerSecurityW';
{$ELSE}
type WTSLISTENERCONFIG = WTSLISTENERCONFIGA;

function WTSQueryListenerConfig(hServer: HANDLE; pReserved: PVOID;
  Reserved: DWORD; pListenerName: LPSTR;
  pBuffer: PWTSLISTENERCONFIGA): WINBOOL; stdcall;
  external wtsapi32_dll name 'WTSQueryListenerConfigA';

function WTSCreateListener(hServer: HANDLE; pReserved: PVOID;
  Reserved: DWORD; pListenerName: LPSTR; pBuffer: PWTSLISTENERCONFIGA;
  flag: DWORD): WINBOOL; stdcall;
  external wtsapi32_dll name 'WTSCreateListenerA';

function WTSSetListenerSecurity(hServer: HANDLE; pReserved: PVOID;
  Reserved: DWORD; pListenerName: LPSTR;
  SecurityInformation: SECURITY_INFORMATION;
  pSecurityDescriptor: PSECURITY_DESCRIPTOR): WINBOOL; stdcall;
  external wtsapi32_dll name 'WTSSetListenerSecurityA';

function WTSGetListenerSecurity(hServer: HANDLE; pReserved: PVOID;
  Reserved: DWORD; pListenerName: LPSTR;
  SecurityInformation: SECURITY_INFORMATION;
  pSecurityDescriptor: PSECURITY_DESCRIPTOR; nLength: DWORD;
  lpnLengthNeeded: LPDWORD): WINBOOL; stdcall;
  external wtsapi32_dll name 'WTSGetListenerSecurityA';
{$ENDIF}

type PWTSLISTENERCONFIG = ^WTSLISTENERCONFIG;

function WTSEnableChildSessions(bEnable: WINBOOL): WINBOOL; stdcall;
  external wtsapi32_dll;

function WTSIsChildSessionsEnabled(pbEnabled: PWINBOOL): WINBOOL; stdcall;
  external wtsapi32_dll;

function WTSGetChildSessionId(pSessionId: PULONG): WINBOOL; stdcall;
  external wtsapi32_dll;


implementation

end.

